(*
 * Coordinates.m
 *
 * Andrew D. Lewis
 * California Institute of Technology
 * 19 April 1996
 * Last updated: 9 February 1999
 *
 * Mathematica for changing coordinates.
 *
 *)

BeginPackage["Coordinates`"]

TransformMetric::usage = "TransformMetric[g,x,xp,xofxp,Deriv] computes
the components of the metric g in the coordinates xp.  Here g are the
components in coordinates x, and xofxp gives x as a function of xp.";

TransformOneForm::usage = "TransformOneForm[omega,x,xp,xofxp,Deriv]
computes the components of the one-form omega in the coordinates xp.
Here omega are the components in coordinates x, and xofxp gives x as a function
of xp.";

TransformVectorField::usage = "TransformVectorField[X,x,xp,xofxp,xpofx,Deriv]
computes the components of the vector field X in terms of the coordinates
xp.  Here X are the components in coordinates x, xofxp gives x as a function
of xp, and xpofx gives xp as a function of x.";

TransformConnection::usage = "TransformConnection[Conn,x,xp,xofxp,xpofx,Deriv]
computes the Christoffel symbols of the vector field X in terms of the
coordinates xp.  Here Conn are the Christoffel symbols in coordinates x, xofxp
gives x as a function of xp, and xpofx gives xp as a function of x.";

Begin["`Private`"]

BeginPackage["DerivativeOptions`"]

Options[TransformMetric] = {Deriv->Explicit}
TransformMetric[g_, x_, xp_, xofxp_, opts___Rule] :=
                Module[{opt1,local,DerivOpts},
(*
 * Gets the options for TransformMetric.
 *)
  opt1 = Deriv /. {opts} /. Options[TransformMetric];
  DerivOpts = {Explicit, Implicit};
  If[!MemberQ[DerivOpts, opt1],
    local = Indeterminate;
    Print["TransformMetric: unknown option: ", opt1]
  ];
  If[opt1 == Explicit, local = TransformMetric1[g,x,xp,xofxp]];
  If[opt1 == Implicit, local = TransformMetric2[g,x,xp,xofxp]];
  local
  ];

TransformMetric1[g_, x_, xp_, xofxp_] := Module[{temp},
(*
 * Transforms metric from coordinates x to coordinates xp.
 * Differentiation is explicit.
 *)
  temp = Table[Sum[Sum[g[[k,l]] D[xofxp[[k]],xp[[i]]] D[xofxp[[l]],xp[[j]]],
        {l,Length[x]}], {k,Length[x]}], {i,Length[xp]}, {j,Length[xp]}];
  Do[temp = temp /. x[[i]] -> xofxp[[i]], {i,Length[x]-1}];
  temp /. x[[Length[x]]] -> xofxp[[Length[x]]]
  ];

TransformMetric2[g_, x_, xp_, xofxp_] := Module[{temp},
(*
 * Transforms metric from coordinates x to coordinates xp.
 * Differentiation is implicit.
 *)
  temp = Table[Sum[Sum[g[[k,l]] Dt[xofxp[[k]],xp[[i]]] Dt[xofxp[[l]],xp[[j]]],
        {l,Length[x]}], {k,Length[x]}], {i,Length[xp]}, {j,Length[xp]}];
  Do[temp = temp /. x[[i]] -> xofxp[[i]], {i,Length[x]-1}];
  temp /. x[[Length[x]]] -> xofxp[[Length[x]]]
  ];

Options[TransformOneForm] = {Deriv->Explicit}
TransformOneForm[omega_, x_, xp_, xofxp_, opts___Rule] :=
                 Module[{opt1,local,DerivOpts},
(*
 * Get options for TransformOneForm.
 *)
  opt1 = Deriv /. {opts} /. Options[TransformOneForm];
  DerivOpts = {Explicit, Implicit};
  If[!MemberQ[DerivOpts, opt1],
    local = Indeterminate;
    Print["TransformOneForm: unknown option: ", opt1]
  ];
  If[opt1 == Explicit, local = TransOF1[omega,x,xp,xofxp]];
  If[opt1 == Implicit, local = TransOF2[omega,x,xp,xofxp]];
  local
  ];

TransOF1[omega_, x_, xp_, xofxp_] := Module[{temp},
(*
 * Transform one-form from x to xp with explicit derivatives.
 *)
  temp = Table[Sum[D[xofxp[[j]],xp[[i]]] omega[[j]],{j,Length[x]}],
        {i,Length[xp]}];
  Do[temp = temp /. x[[i]] -> xofxp[[i]], {i,Length[x]-1}];
  temp /. x[[Length[x]]] -> xofxp[[Length[x]]]
  ];

TransOF2[omega_, x_, xp_, xofxp_] := Module[{temp},
(*
 * Transform one-form from x to xp with implicit derivatives.
 *)
  temp = Table[Sum[Dt[xofxp[[j]],xp[[i]]] omega[[j]],{j,Length[x]}],
        {i,Length[xp]}];
  Do[temp = temp /. x[[i]] -> xofxp[[i]], {i,Length[x]-1}];
  temp /. x[[Length[x]]] -> xofxp[[Length[x]]]
  ];

Options[TransformVectorField] = {Deriv->Explicit}
TransformVectorField[X_, x_, xp_, xofxp_, xpofx_, opts___Rule] :=
                     Module[{opt1,local,DerivOpts},
(*
 * Get options for TransformVectorField.
 *)
  opt1 = Deriv /. {opts} /. Options[TransformVectorField];
  DerivOpts = {Explicit, Implicit};
  If[!MemberQ[DerivOpts, opt1],
    local = Indeterminate;
    Print["TransformVectorField: unknown option: ", opt1]
  ];
  If[opt1 == Explicit, local = TransVF1[X,x,xp,xofxp,xpofx]];
  If[opt1 == Implicit, local = TransVF2[X,x,xp,xofxp,xpofx]];
  local
  ];

TransVF1[X_, x_, xp_, xofxp_, xpofx_] := Module[{temp},
(*
 * This module computes the components of a vector field X in coordinates
 * xp.  Differentiation is explicit
 *)
  temp = Table[Sum[D[xpofx[[i]],x[[j]]] X[[j]], {j, Length[x]}],
               {i, Length[x]}];
  Do[temp = temp/. x[[i]] -> xofxp[[i]], {i,Length[x]}];
  temp
  ];

TransVF2[X_, x_, xp_, xofxp_, xpofx_] := Module[{temp},
(*
 * This module computes the components of a vector field X in coordinates
 * xp.  Differentiation is implicit.
 *)
  temp = Table[Sum[Dt[xpofx[[i]],x[[j]]] X[[j]], {j, Length[x]}],
               {i, Length[x]}];
  Do[temp = temp/. x[[i]] -> xofxp[[i]], {i,Length[x]}];
  temp
  ];

Options[TransformConnection] = {Deriv->Explicit}
TransformConnection[Conn_, x_, xp_, xofxp_, xpofx_, opts___Rule] :=
                    Module[{opt1,local,DerivOpts},
(*
 * Get options for TransformConnection.
 *)
  opt1 = Deriv /. {opts} /. Options[TransformConnection];
  DerivOpts = {Explicit, Implicit};
  If[!MemberQ[DerivOpts, opt1],
    local = Indeterminate;
    Print["TransformConnection: unknown option: ", opt1]
  ];
  If[opt1 == Explicit, local = TransConn1[Conn,x,xp,xofxp,xpofx]];
  If[opt1 == Implicit, local = TransConn2[Conn,x,xp,xofxp,xpofx]];
  local
  ];

TransConn1[Conn_, x_, xp_, xofxp_, xpofx_] := Module[{temp},
(*
 * This modules transforms connection coefficients explicitly.
 *)
  temp = Table[Sum[Conn[[r,p,q]]*D[xofxp[[p]],xp[[i]]]*
               D[xofxp[[q]],xp[[j]]]*D[xpofx[[k]],x[[r]]],{r,Length[x]},
               {p,Length[x]},{q,Length[x]}]+
               Sum[D[xofxp[[p]],xp[[i]],xp[[j]]] D[xpofx[[k]],x[[p]]],
               {p,Length[x]}],{k,Length[x]},{i,Length[x]},{j,Length[x]}];
  Do[temp = temp/. x[[i]] -> xofxp[[i]], {i,Length[x]}];
  temp
];

TransConn2[Conn_, x_, xp_, xofxp_, xpofx_] := Module[{temp},
(*
 * This modules transforms connection coefficients implicitly.
 *)
  temp = Table[Sum[Conn[[r,p,q]]*Dt[xofxp[[p]],xp[[i]]]*
               Dt[xofxp[[q]],xp[[j]]]*Dt[xpofx[[k]],x[[r]]],{r,1,Length[x]},
               {p,1,Length[x]},{q,1,Length[x]}]+
               Sum[Dt[xofxp[[p]],xp[[i]],xp[[j]]] Dt[xpofx[[k]],x[[p]]],
               {p,1,Length[x]}],{k,Length[x]},{i,Length[x]},{j,Length[x]}];
  Do[temp = temp/. x[[i]] -> xofxp[[i]], {i,Length[x]}];
  temp
];

End[]
EndPackage[]
